import * as i0 from '@angular/core';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import { timeout } from 'rxjs/operators';
import * as i1 from '@ionic/angular/standalone';
import * as i2 from '@angular/common/http';

class ApiHelperService {
    constructor(loading, http) {
        this.loading = loading;
        this.http = http;
        // the base api link. Ex: http://localhost/smart_pos/api.php
        // if it is not set, then full url should be used when calling post or get method
        this.BaseURL = '';
        this.EnableLog = false;
    }
    _getAPIURL(functionName) {
        // prepare api url
        let apiURL = this.BaseURL;
        if (apiURL.length > 0) { // if base url is set, then concate with the function name
            if (!apiURL.endsWith('/')) {
                apiURL += '/';
            }
            apiURL += functionName;
        }
        else { // if base url is not set, then full api url is provided
            apiURL = functionName;
        }
        return apiURL;
    }
    _getPostDataOrParams(method, postDataOrParam) {
        if (method == EMethod.POST) {
            let tmp = JSON.stringify(postDataOrParam);
            if (this.EnableLog)
                console.log(tmp); // for testing purpose
            return tmp;
        }
        else {
            if (postDataOrParam.length > 0) {
                return '/' + postDataOrParam.join('/');
            }
            else {
                return "";
            }
        }
    }
    async _request_api(method, functionName, postDataOrParam, time_out) {
        return new Promise((resolve, reject) => {
            // choose to use post or get method
            // let ret: Promise<any>;
            let obs;
            let sub;
            if (method == EMethod.POST) {
                // ret = this.http.post(this._getAPIURL(functionName), this._getPostDataOrParams(method, postDataOrParam)).toPromise();
                obs = this.http.post(this._getAPIURL(functionName), this._getPostDataOrParams(method, postDataOrParam)).pipe(timeout(time_out * 1000));
            }
            else {
                // ret = this.http.get(this._getAPIURL(functionName) + this._getPostDataOrParams(method, postDataOrParam)).toPromise();
                obs = this.http.get(this._getAPIURL(functionName) + this._getPostDataOrParams(method, postDataOrParam)).pipe(timeout(time_out * 1000));
            }
            // success or reject promise
            let tmp1 = new Date();
            sub = obs.subscribe({
                next: (res) => {
                    if (this.EnableLog) {
                        let tmp2 = new Date();
                        console.log((tmp2.getTime() - tmp1.getTime()) / 1000);
                    }
                    // clearTimeout(timer);
                    resolve(res);
                },
                error: (e) => {
                    // clearTimeout(timer);
                    if (this.EnableLog) {
                        console.log("Calling URL", functionName);
                    }
                    reject(e);
                }
            });
            // promise time out
            // let timer: any;
            // if (time_out > 0) {
            //   timer = window.setTimeout(() => {
            //     sub.unsubscribe();
            //     let error = {
            //       name: 'Request Timeout',
            //       message: 'The server is not responed! Make sure your network is available and try again.'
            //     }
            //     reject(error);
            //   }, time_out * 1000);
            // }
        });
        // try {
        //   let res = await ret;
        //   clearTimeout(timer);
        //   return res;
        // } catch (e) {
        //   clearTimeout(timer);
        //   throw e;
        // }
    }
    _request_obs(method, functionName, postDataOrParam) {
        return new Observable(subscriber => {
            let obs;
            if (method == EMethod.POST) {
                obs = this.http.post(this._getAPIURL(functionName), this._getPostDataOrParams(method, postDataOrParam));
            }
            else {
                obs = this.http.get(this._getAPIURL(functionName) + this._getPostDataOrParams(method, postDataOrParam));
            }
            let tmp1 = new Date();
            let sub = obs.subscribe({
                next: (res) => {
                    if (this.EnableLog) {
                        let tmp2 = new Date();
                        console.log((tmp2.getTime() - tmp1.getTime()) / 1000);
                    }
                    subscriber.next(res);
                    subscriber.complete();
                },
                error: (e) => {
                    subscriber.error(e);
                    subscriber.complete();
                }
            });
            return {
                unsubscribe() {
                    sub.unsubscribe();
                }
            };
        });
    }
    post(functionName_or_fullURL, data, time_out = 60) {
        return this._request_api(EMethod.POST, functionName_or_fullURL, data, time_out);
    }
    postObs(functionName_or_fullURL, data) {
        return this._request_obs(EMethod.POST, functionName_or_fullURL, data);
    }
    get(functionName_or_fullURL, params = [], time_out = 60) {
        return this._request_api(EMethod.GET, functionName_or_fullURL, params, time_out);
    }
    getObs(functionName_or_fullURL, params = []) {
        return this._request_obs(EMethod.GET, functionName_or_fullURL, params);
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: ApiHelperService, deps: [{ token: i1.LoadingController }, { token: i2.HttpClient }], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: ApiHelperService, providedIn: 'root' }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: ApiHelperService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: () => [{ type: i1.LoadingController }, { type: i2.HttpClient }] });
var EMethod;
(function (EMethod) {
    EMethod[EMethod["GET"] = 0] = "GET";
    EMethod[EMethod["POST"] = 1] = "POST";
})(EMethod || (EMethod = {}));

class UiHelperService {
    constructor(loader, alertCtrl, toastCtrl) {
        this.loader = loader;
        this.alertCtrl = alertCtrl;
        this.toastCtrl = toastCtrl;
    }
    // loading component
    async showLoading(text = 'Please wait...', duration = 30) {
        // dismiss previous loading if exists
        // if (this._loader) {
        //   this._loader.dismiss();
        // }
        this._isLoading = true;
        this._loader = await this.loader.create({
            message: text,
            duration: duration * 1000
        });
        await this._loader.present();
    }
    hideLoading() {
        if (this._isLoading && this._loader) {
            this._loader.dismiss();
            this._isLoading = false;
        }
    }
    // alert component
    alert(title, message) {
        return new Promise((resolve) => {
            this.alertCtrl.create({
                header: title,
                message: message,
                buttons: [{
                        text: "OK"
                    }],
                backdropDismiss: false,
            }).then(ctrl => {
                ctrl.onDidDismiss().then(() => {
                    resolve();
                });
                ctrl.present();
            });
        });
    }
    alertError(e, logError = false) {
        return new Promise((resolve) => {
            if (logError) {
                console.log(e);
            }
            let title;
            let msg;
            if (e.error && e.error.name !== undefined) {
                title = e.error.name;
                msg = e.error.message;
            }
            else {
                title = e.name;
                if (e.error && e.error.message !== undefined)
                    msg = e.error.message;
                else
                    msg = e.message;
            }
            this.alertCtrl.create({
                header: title,
                message: msg,
                buttons: [{
                        text: 'OK',
                        handler: () => {
                            resolve();
                        }
                    }]
            }).then(ctrl => {
                ctrl.present();
            });
        });
    }
    alertYesNo(title, message, yesText = "Yes", noText = "No") {
        return new Promise((resolve) => {
            this.alertCtrl.create({
                header: title,
                message: message,
                buttons: [{
                        text: noText,
                        role: 'cancel',
                        handler: () => {
                            resolve(false);
                        }
                    }, {
                        text: yesText,
                        handler: () => {
                            resolve(true);
                        }
                    }]
            }).then(ctrl => {
                ctrl.present();
            });
        });
    }
    // toast compoent
    showToast(message, duration = 3000, position = EToastPosition.Bottom) {
        let opt = {
            message: message,
            position: position
        };
        if (duration > 0) {
            opt.duration = duration;
        }
        this._isShowToast = true;
        this.toastCtrl.create(opt).then(ctrl => {
            ctrl.present().then(() => {
                if (!this._isShowToast) {
                    ctrl.dismiss();
                }
            });
        });
    }
    hideToast() {
        this._isShowToast = false;
        this.toastCtrl.dismiss();
    }
    /**
     * Shows toast when data is saved. Used this, so we don't have to write the same message multiple times.
     */
    showToast_Saved() {
        this.showToast("Data saved");
    }
    /**
     * Delays promise
     * @param milisecond number of miliseconds to delay
     * @returns promise that resolved after delay
     */
    delay(milisecond) {
        return new Promise((resolve) => {
            window.setTimeout(() => {
                resolve();
            }, milisecond);
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: UiHelperService, deps: [{ token: i1.LoadingController }, { token: i1.AlertController }, { token: i1.ToastController }], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: UiHelperService, providedIn: 'root' }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: UiHelperService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: () => [{ type: i1.LoadingController }, { type: i1.AlertController }, { type: i1.ToastController }] });
var EToastPosition;
(function (EToastPosition) {
    EToastPosition["Bottom"] = "bottom";
    EToastPosition["Middle"] = "middle";
    EToastPosition["Top"] = "top";
})(EToastPosition || (EToastPosition = {}));

class DatetimeHelperService {
    constructor() { }
    /**
     * Gets ISO date string with time-zone offset
     * @param myDate date to be converted
     * @returns ISO date string
     */
    getISODate(myDate = new Date()) {
        var tzo = -myDate.getTimezoneOffset(), dif = tzo >= 0 ? '+' : '-', pad = function (num) {
            var norm = Math.floor(Math.abs(num));
            return (norm < 10 ? '0' : '') + norm;
        };
        return myDate.getFullYear() +
            '-' + pad(myDate.getMonth() + 1) +
            '-' + pad(myDate.getDate()) +
            'T' + pad(myDate.getHours()) +
            ':' + pad(myDate.getMinutes()) +
            ':' + pad(myDate.getSeconds()) +
            dif + pad(tzo / 60) +
            ':' + pad(tzo % 60);
    }
    /** Date different */
    DateDiff(minDate, maxDate) {
        let res = {
            second: 0,
            minute: 0,
            hour: 0,
            day: 0,
            month: 0,
            year: 0
        };
        let value = maxDate.getTime() - minDate.getTime();
        res.second = Math.floor(value / 1000);
        res.minute = Math.floor(res.second / 60);
        res.hour = Math.floor(res.minute / 60);
        res.day = Math.floor(res.hour / 24);
        res.year = maxDate.getFullYear() - minDate.getFullYear();
        res.month = maxDate.getMonth() - minDate.getMonth() + res.year * 12;
        return res;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: DatetimeHelperService, deps: [], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: DatetimeHelperService, providedIn: "root" }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: DatetimeHelperService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: "root"
                }]
        }], ctorParameters: () => [] });

class ScreenService {
    constructor(pl) {
        this.pl = pl;
        /**
         * Size of screen
         */
        this.size = {
            xs: false,
            sm: false,
            md: false,
            lg: false,
            xl: false,
            sm_down: false,
            sm_up: false,
            md_up: false,
            md_down: false,
            lg_up: false,
            lg_down: false
        };
        this.listenToResize();
    }
    listenToResize() {
        this.resetScreenSize();
        this.pl.resize.subscribe(() => {
            this.resetScreenSize();
        });
    }
    resetScreenSize() {
        this.size.xs = false;
        this.size.sm = false;
        this.size.md = false;
        this.size.lg = false;
        this.size.xl = false;
        this.size.sm_down = false;
        this.size.sm_up = false;
        this.size.md_down = false;
        this.size.md_up = false;
        this.size.lg_down = false;
        this.size.lg_up = false;
        let w = this.pl.width();
        if (w <= 576) {
            this.size.xs = true;
            this.size.sm_down = true;
            this.size.md_down = true;
            this.size.lg_down = true;
        }
        else if (w < 768) {
            this.size.sm = true;
            this.size.sm_up = true;
            this.size.md_down = true;
            this.size.lg_down = true;
        }
        else if (w < 992) {
            this.size.md = true;
            this.size.lg_down = true;
            this.size.sm_up = true;
            this.size.md_up = true;
        }
        else if (w < 1200) {
            this.size.lg = true;
            this.size.sm_up = true;
            this.size.md_up = true;
            this.size.lg_up = true;
        }
        else {
            this.size.xl = true;
            this.size.sm_up = true;
            this.size.md_up = true;
            this.size.lg_up = true;
        }
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: ScreenService, deps: [{ token: i1.Platform }], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: ScreenService, providedIn: 'root' }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.13", ngImport: i0, type: ScreenService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: () => [{ type: i1.Platform }] });

/*
 * Public API Surface of cck-ionic-helper
 */

/**
 * Generated bundle index. Do not edit.
 */

export { ApiHelperService, DatetimeHelperService, EToastPosition, ScreenService, UiHelperService };
//# sourceMappingURL=cck-ionic-helper.mjs.map
